#!/usr/bin/env python

'''
`scipy_linkage_matrix_to_dendropy_tree` tests for
`alifedata-phyloinformatics-convert` package.
'''

import dendropy
from dendropy.calculate import treecompare
import numpy as np

import alifedata_phyloinformatics_convert as apc


def test():

    # from https://stackoverflow.com/a/40983611/17332200
    linkage_matrix = np.array([
        [7.0, 9.0, 0.3, 2.0],
        [4.0, 6.0, 0.5, 2.0],
        [5.0, 12.0, 0.5, 3.0],
        [2.0, 13.0, 0.53851648, 4.0],
        [3.0, 14.0, 0.58309519, 5.0],
        [1.0, 15.0, 0.64031242, 6.0],
        [10.0, 11.0, 0.72801099, 3.0],
        [8.0, 17.0, 1.2083046, 4.0],
        [0.0, 16.0, 1.5132746, 7.0],
        [18.0, 19.0, 1.92353841, 11.0],
    ])

    tree = apc.scipy_linkage_matrix_to_dendropy_tree(linkage_matrix)

    assert sum(1 for __ in tree.leaf_node_iter()) == 11

    def node(cluster_id: float) -> dendropy.Node:
        return next(
            node for node in tree if node.cluster_id == cluster_id
        )

    assert {n.cluster_id for n in node(11).child_nodes()} == {7., 9.}
    assert node(7).edge_length == 0.3 / 2
    assert node(9).edge_length == 0.3 / 2

    assert {n.cluster_id for n in node(12).child_nodes()} == {4., 6.}
    assert node(4).edge_length == 0.5 / 2
    assert node(6).edge_length == 0.5 / 2

    assert {n.cluster_id for n in node(17).child_nodes()} == {10., 11.}
    assert node(10).edge_length == 0.72801099 / 2
    assert node(11).edge_length == 0.72801099 / 2

    assert {n.cluster_id for n in node(20).child_nodes()} == {18., 19.}
    assert node(18).edge_length == 1.92353841 / 2
    assert node(19).edge_length == 1.92353841 / 2
    assert node(20).parent_node is None


def test_leaf_taxon_labels():

    # from https://stackoverflow.com/a/40983611/17332200
    linkage_matrix = np.array([
        [7.0, 9.0, 0.3, 2.0],
        [4.0, 6.0, 0.5, 2.0],
        [5.0, 12.0, 0.5, 3.0],
        [2.0, 13.0, 0.53851648, 4.0],
        [3.0, 14.0, 0.58309519, 5.0],
        [1.0, 15.0, 0.64031242, 6.0],
        [10.0, 11.0, 0.72801099, 3.0],
        [8.0, 17.0, 1.2083046, 4.0],
        [0.0, 16.0, 1.5132746, 7.0],
        [18.0, 19.0, 1.92353841, 11.0],
    ])

    tree1 = apc.scipy_linkage_matrix_to_dendropy_tree(
        linkage_matrix,
        [*range(11)],
    )

    for node in tree1.leaf_node_iter():
        assert node.taxon is not None
        assert node.taxon.label is not None
        assert node.cluster_id == node.taxon.label

    for node in tree1.preorder_internal_node_iter():
        assert node.taxon is None

    tree2 = apc.scipy_linkage_matrix_to_dendropy_tree(
        linkage_matrix,
        [*range(11)],
    )

    tree2.migrate_taxon_namespace(tree1.taxon_namespace)

    assert treecompare.symmetric_difference(tree1, tree2) == 0



#!/usr/bin/env python

'''
`dendropy_tree_to_alife_dataframe` tests for
`alifedata-phyloinformatics-convert` package.
'''

import dendropy
from dendropy.calculate import treecompare
from os.path import dirname, realpath

import alifedata_phyloinformatics_convert as apc


def test_newick1():

    original_tree = dendropy.Tree.get(
        path=f'{dirname(realpath(__file__))}/assets/APG_Angiosperms.newick',
        schema='newick',
    )

    converted_df = apc.dendropy_tree_to_alife_dataframe(original_tree)
    reconverted_tree = apc.alife_dataframe_to_dendropy_tree(converted_df)
    reconverted_df = apc.dendropy_tree_to_alife_dataframe(reconverted_tree)

    assert len(converted_df.compare(reconverted_df)) == 0
    assert len(converted_df) == len(reconverted_df)
    assert len(converted_df) == len(original_tree.nodes())
    assert len(reconverted_df) == len(original_tree.nodes())
    assert len(reconverted_df) == len(reconverted_tree.nodes())

    assert len(reconverted_tree.nodes()) == len(original_tree.nodes())
    assert len(reconverted_tree.leaf_nodes()) \
        == len(original_tree.leaf_nodes())
    assert original_tree.as_ascii_plot() == reconverted_tree.as_ascii_plot()
    assert str(original_tree) == str(reconverted_tree)


def test_newick2():

    original_tree = dendropy.Tree.get(
        path=f'{dirname(realpath(__file__))}/assets/APG_Angiosperms.newick',
        schema='newick',
    )
    original_tree.resolve_polytomies()
    original_tree.suppress_unifurcations()
    original_tree.collapse_basal_bifurcation()

    converted_df = apc.dendropy_tree_to_alife_dataframe(original_tree)
    reconverted_tree = apc.alife_dataframe_to_dendropy_tree(converted_df)
    reconverted_df = apc.dendropy_tree_to_alife_dataframe(reconverted_tree)

    assert len(converted_df.compare(reconverted_df)) == 0
    assert len(converted_df) == len(reconverted_df)
    assert len(converted_df) == len(original_tree.nodes())
    assert len(reconverted_df) == len(original_tree.nodes())
    assert len(reconverted_df) == len(reconverted_tree.nodes())

    reconverted_tree.migrate_taxon_namespace(original_tree.taxon_namespace)

    assert original_tree.as_ascii_plot() == reconverted_tree.as_ascii_plot()
    assert str(original_tree) == str(reconverted_tree)
    assert treecompare.symmetric_difference(
        original_tree,
        reconverted_tree,
    ) == 0


def test_nexml():

    script_directory = dirname(realpath(__file__))
    original_tree = dendropy.Tree.get(
        path=f'{script_directory}/assets/pythonidae.annotated.nexml',
        schema='nexml',
    )
    converted_df = apc.dendropy_tree_to_alife_dataframe(original_tree)
    reconverted_tree = apc.alife_dataframe_to_dendropy_tree(
        converted_df,
    )

    # ensure taxons can be compared between trees
    reconverted_tree.migrate_taxon_namespace(original_tree.taxon_namespace)

    assert treecompare.symmetric_difference(
        original_tree,
        reconverted_tree,
    ) == 0



import dendropy
from iterpop import iterpop as ip
from lyncs_utils import keydefaultdict
from nanto import isanan, nantonone
import pandas as pd
import string
import typing


def _parse_ancestor_list(raw: str) -> typing.List[int]:
    assert not any(
        c in raw for c in string.whitespace
    ), "Whitespace separated ancestor list not supported."
    try:
        ancestor_list = eval(raw)
        if ancestor_list == [None]:
            return []
        else:
            assert None not in ancestor_list
            return ancestor_list
    except NameError:
        # if ancestor list contains a placeholder string like NONE,
        # it will trigger a NameError when eval'ed
        return []


def alife_dataframe_to_dendropy_trees(
    df: pd.DataFrame,
    setup_edge_lengths: bool = False,
) -> typing.List[dendropy.Tree]:

    df = df.copy()

    df['parsed_ancestor_list'] \
        = df['ancestor_list'].apply(_parse_ancestor_list)

    # maps id to node
    def setup_node(id: int) -> dendropy.Node:
        res = dendropy.Node()
        res.id = id
        return res
    nodes = keydefaultdict(setup_node)

    # maps id to origin time
    root_nodes = []

    for __, row in df.iterrows():
        node = nodes[row['id']]

        node.origin_time = nantonone(row.get('origin_time', default=None))
        node.label = row.get('label', default=None)
        taxon_label = row.get('taxon_label', default=None)
        if taxon_label not in ('None', None):
            node.taxon = dendropy.Taxon(label=taxon_label)
        node.edge_length = nantonone(row.get('edge_length', None))

        if row['parsed_ancestor_list']:
            ancestor_id = ip.popsingleton(row['parsed_ancestor_list'])
            nodes[ancestor_id].add_child(node)
        else:
            root_nodes.append(node)

    # set up edge lengths
    if setup_edge_lengths:
        for id, parent_node in nodes.items():
            for child_node in parent_node.child_node_iter():
                if child_node.edge_length is None and None not in (
                    getattr(child_node, 'origin_time', None),
                    getattr(parent_node, 'origin_time', None),
                ):
                    assert child_node.origin_time >= parent_node.origin_time
                    child_node.edge_length \
                        = child_node.origin_time - parent_node.origin_time

        for root_node in root_nodes:
            if (
                root_node.edge_length is None
                and getattr(root_node, 'origin_time', None) is not None
            ):
                assert not isanan(root_node.origin_time)
                root_node.edge_length = root_node.origin_time

    return([
        dendropy.Tree(seed_node=root_node)
        for root_node in root_nodes
    ])



import dendropy
from iterpop import iterpop as ip
import pandas as pd
import typing

from .alife_dataframe_to_dendropy_trees \
    import alife_dataframe_to_dendropy_trees


def alife_dataframe_to_dendropy_tree(
    df: pd.DataFrame,
    setup_edge_lengths: bool = False,
) -> typing.Optional[dendropy.Tree]:
    return ip.poursingleton(
        alife_dataframe_to_dendropy_trees(
            df,
            setup_edge_lengths=setup_edge_lengths,
        ),
    )



import click
import dendropy
import pandas as pd

from .alife_dataframe_to_dendropy_tree import alife_dataframe_to_dendropy_tree
from .dendropy_tree_to_alife_dataframe import dendropy_tree_to_alife_dataframe


@click.group()
def cli():
    pass


@cli.command(
    help='convert standard alife phylogeny data to phloinformatics format',
)
@click.option(
    '--input-file',
    default='-',
    help='phyloinformatics data file path; default stdin',
    type=click.File('r'),
)
@click.option(
    '--input-schema',
    help=(
        'phyloinformatics data format schema; '
        'options include newick, nexml, and nexus'
    ),
    required=True,
)
@click.option(
    '--output-file',
    help='alife data file path; default stdout',
    default='-',
    type=click.File('w'),
)
@click.option(
    '--output-format',
    help='alife data file format; default csv',
    default='csv',
)
def toalifedata(
    input_file,
    input_schema,
    output_file,
    output_format,
):

    tree = dendropy.Tree.get(
        file=input_file,
        schema=input_schema,
    )

    converted_df = dendropy_tree_to_alife_dataframe(tree)

    {
        'csv': lambda file: converted_df.to_csv(file, index=False),
        'json': converted_df.to_json,
        'html': converted_df.to_html,
        'excel': converted_df.to_excel,
        'hdf': converted_df.to_hdf,
        'feather': converted_df.to_feather,
        'parquet': converted_df.to_parquet,
        'stata': converted_df.to_stata,
        'pickle': converted_df.to_pickle,
        'sql': converted_df.to_sql,
        'gbq': converted_df.to_gbq,
    }[output_format](
        output_file,
    )


@cli.command(
    help='convert phloinformatics data to standard alife phylogeny format',
)
@click.option(
    '--input-file',
    default='-',
    help='alife data file path; default stdin',
    type=click.File('r'),
)
@click.option(
    '--input-format',
    help='alife data file format; default csv',
    default='csv',
)
@click.option(
    '--output-file',
    help='phyloinformatics data file path; default stdout',
    default='-',
    type=click.File('w'),
)
@click.option(
    '--output-schema',
    help=(
        'phyloinformatics data format schema; '
        'options include newick, nexml, and nexus'
    ),
    required=True,
)
def fromalifedata(
    input_file,
    input_format,
    output_file,
    output_schema,
):
    df = {
        'csv': pd.read_csv,
        'fwf': pd.read_fwf,
        'json': pd.read_json,
        'html': pd.read_html,
        'excel': pd.read_excel,
        'hdf': pd.read_hdf,
        'feather': pd.read_feather,
        'parquet': pd.read_parquet,
        'orc': pd.read_orc,
        'stata': pd.read_stata,
        'sass': pd.read_sas,
        'spss': pd.read_spss,
        'pickle': pd.read_pickle,
        'sql': pd.read_sql,
        'gbq': pd.read_gbq,
    }[input_format](
        input_file,
    )

    converted_tree = alife_dataframe_to_dendropy_tree(
        df,
        setup_edge_lengths=True,
    )

    converted_tree.write(
        file=output_file,
        schema=output_schema,
    )



import dendropy
from nanto import isanan
import opytional as opyt
import pandas as pd


def dendropy_tree_to_alife_dataframe(tree: dendropy.Tree) -> pd.DataFrame:

    # set up node origin times if any edge lengths set
    if any(node.edge_length is not None for node in tree):
        if not hasattr(tree.seed_node, 'origin_time'):
            tree.seed_node.origin_time = opyt.or_value(
                tree.seed_node.edge_length,
                0
            )
        else:
            assert tree.seed_node.origin_time is not None
        for node in tree:
            parent = node.parent_node
            if parent is not None and not hasattr(node, 'origin_time'):
                if None not in (
                    getattr(parent, 'origin_time', None),
                    node.edge_length,
                ) and not isanan(parent.origin_time):
                    node.origin_time = parent.origin_time + node.edge_length

    # attach ids to nodes, if needed
    for fallback_id, node in enumerate(tree):
        if not hasattr(node, 'id'):
            node.id = fallback_id
        else:
            assert isinstance(node.id, int)

    return pd.DataFrame.from_records([
        {
            'id': node.id,
            'ancestor_list': str(
                [opyt.apply_if(node.parent_node, lambda x: x.id)]
            ),
            'origin_time': getattr(node, 'origin_time', None),
            'edge_length': node.edge_length,
            'label': node.label,
            'taxon_label': opyt.apply_if(node.taxon, lambda x: x.label),
        }
        for node in tree
    ])



#!/usr/bin/env python

'''
`alife_dataframe_to_dendropy_tree` tests for
`alifedata-phyloinformatics-convert` package.
'''

from os.path import dirname, realpath
import pandas as pd

import alifedata_phyloinformatics_convert as apc


def _setify(df):
    return set(
        (
            int(row['id']),
            row['ancestor_list'].upper(),
            int(row['origin_time']),
        )
        for __, row in df.iterrows()
    )


def _setify_no_origin_time(df):
    return set(
        (
            int(row['id']),
            row['ancestor_list'].upper(),
        )
        for __, row in df.iterrows()
    )


def test():
    original_df = pd.read_csv(
        f'{dirname(realpath(__file__))}/assets/alifedata.csv',
    )
    converted_tree = apc.alife_dataframe_to_dendropy_tree(original_df)
    reconverted_df = apc.dendropy_tree_to_alife_dataframe(converted_tree)

    assert _setify(original_df) == _setify(reconverted_df)


def test_minimal():
    original_df = pd.read_csv(
        f'{dirname(realpath(__file__))}/assets/alifedata_minimal.csv'
    )
    converted_tree = apc.alife_dataframe_to_dendropy_tree(original_df)
    reconverted_df = apc.dendropy_tree_to_alife_dataframe(converted_tree)

    assert _setify_no_origin_time(original_df) \
        == _setify_no_origin_time(reconverted_df)


def test_empty():
    df = pd.read_csv(
        f'{dirname(realpath(__file__))}/assets/alifedata_empty.csv'
    )
    assert apc.alife_dataframe_to_dendropy_tree(df) is None


def test_minimal_empty():
    df = pd.read_csv(
        f'{dirname(realpath(__file__))}/assets/alifedata_minimal_empty.csv'
    )
    assert apc.alife_dataframe_to_dendropy_tree(df) is None



"""Top-level package for alifedata-phyloinformatics-convert."""

__author__ = """Matthew Andres Moreno"""
__email__ = 'm.more500@gmail.com'
__version__ = '0.3.0'

from .alife_dataframe_to_dendropy_tree import alife_dataframe_to_dendropy_tree
from .alife_dataframe_to_dendropy_trees \
    import alife_dataframe_to_dendropy_trees
from .dendropy_tree_to_alife_dataframe import dendropy_tree_to_alife_dataframe
from .dendropy_tree_to_scipy_linkage_matrix \
    import dendropy_tree_to_scipy_linkage_matrix
from .scipy_linkage_matrix_to_dendropy_tree \
    import scipy_linkage_matrix_to_dendropy_tree

# adapted from https://stackoverflow.com/a/31079085
__all__ = [
    'alife_dataframe_to_dendropy_tree',
    'alife_dataframe_to_dendropy_trees',
    'dendropy_tree_to_alife_dataframe',
    'dendropy_tree_to_scipy_linkage_matrix',
    'scipy_linkage_matrix_to_dendropy_tree',
]
